"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var mockFactory_1 = require("../test/mockFactory");
var resourceService_1 = require("./resourceService");
jest.mock("@azure/arm-resources");
var arm_resources_1 = require("@azure/arm-resources");
describe("Resource Service", function () {
    beforeAll(function () {
        arm_resources_1.ResourceManagementClient.prototype.resourceGroups = {
            createOrUpdate: jest.fn(),
            deleteMethod: jest.fn(),
        };
        arm_resources_1.ResourceManagementClient.prototype.deployments = {
            deleteMethod: jest.fn()
        };
    });
    it("throws error with empty credentials", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        delete sls.variables["azureCredentials"];
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        expect(function () { return new resourceService_1.ResourceService(sls, options); }).toThrowError("Azure Credentials has not been set in ResourceService");
    });
    it("initializes a resource service", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        expect(function () { return new resourceService_1.ResourceService(sls, options); }).not.toThrowError();
    });
    it("deploys a resource group", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        var location = "West Us";
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.service.provider["location"] = location;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deployResourceGroup();
        expect(arm_resources_1.ResourceManagementClient.prototype.resourceGroups.createOrUpdate)
            .toBeCalledWith(resourceGroup, { location: location });
    });
    it("deletes a deployment", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        var deploymentName = "myDeployment";
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.service.provider["deploymentName"] = deploymentName;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deleteDeployment();
        expect(arm_resources_1.ResourceManagementClient.prototype.deployments.deleteMethod)
            .toBeCalledWith(resourceGroup, deploymentName);
    });
    it("deletes a resource group", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deleteResourceGroup();
        expect(arm_resources_1.ResourceManagementClient.prototype.resourceGroups.deleteMethod)
            .toBeCalledWith(resourceGroup);
    });
});
