"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ms_rest_js_1 = require("@azure/ms-rest-js");
var js_yaml_1 = __importDefault(require("js-yaml"));
var serverless_1 = __importDefault(require("serverless"));
function getAttribute(object, prop, defaultValue) {
    if (object && object[prop]) {
        return object[prop];
    }
    return defaultValue;
}
var MockFactory = /** @class */ (function () {
    function MockFactory() {
    }
    MockFactory.createTestServerless = function (config) {
        var sls = new serverless_1.default(config);
        sls.utils = getAttribute(config, "utils", MockFactory.createTestUtils());
        sls.cli = getAttribute(config, "cli", MockFactory.createTestCli());
        sls.pluginManager = getAttribute(config, "pluginManager", MockFactory.createTestPluginManager());
        sls.variables = getAttribute(config, "variables", MockFactory.createTestVariables());
        sls.service = getAttribute(config, "service", MockFactory.createTestService());
        sls.service.getFunction = jest.fn(function (functionName) { return sls.service["functions"][functionName]; });
        return sls;
    };
    MockFactory.createTestServerlessOptions = function () {
        return {
            extraServicePath: null,
            function: null,
            noDeploy: null,
            region: null,
            stage: null,
            watch: null,
        };
    };
    MockFactory.createTestArmSdkResponse = function (model, statusCode) {
        var response = {
            headers: new ms_rest_js_1.HttpHeaders(),
            request: null,
            status: statusCode,
        };
        var result = __assign({}, model, { _response: response });
        return Promise.resolve(result);
    };
    MockFactory.createTestAuthResponse = function () {
        return {
            credentials: MockFactory.createTestVariables()
                .azureCredentials,
            subscriptions: [
                {
                    id: "azureSubId",
                },
            ],
        };
    };
    MockFactory.createTestAzureCredentials = function () {
        var credentials = {
            getToken: jest.fn(function () {
                var token = {
                    tokenType: "Bearer",
                    accessToken: "ABC123",
                };
                return Promise.resolve(token);
            }),
            signRequest: jest.fn(function (resource) { return Promise.resolve(resource); }),
        };
        // TODO: Reduce usage on tokenCache._entries[0]
        credentials["tokenCache"] = {
            _entries: [{ accessToken: "ABC123" }]
        };
        return credentials;
    };
    MockFactory.createTestAxiosResponse = function (config, responseJson, statusCode) {
        if (statusCode === void 0) { statusCode = 200; }
        var statusText;
        switch (statusCode) {
            case 200:
                statusText = "OK";
                break;
            case 404:
                statusText = "NotFound";
                break;
        }
        var response = {
            config: config,
            data: JSON.stringify(responseJson),
            headers: {
                "Content-Type": "application/json; charset=utf-8",
            },
            status: statusCode,
            statusText: statusText,
        };
        return Promise.resolve(response);
    };
    MockFactory.createTestAzureClientResponse = function (responseJson, statusCode) {
        if (statusCode === void 0) { statusCode = 200; }
        var response = {
            request: new ms_rest_js_1.WebResource(),
            parsedBody: responseJson,
            bodyAsText: JSON.stringify(responseJson),
            headers: new ms_rest_js_1.HttpHeaders(),
            status: statusCode,
        };
        return Promise.resolve(response);
    };
    MockFactory.createTestServerlessYml = function (asYaml, functionMetadata) {
        if (asYaml === void 0) { asYaml = false; }
        var data = {
            provider: {
                name: "azure",
                location: "West US 2"
            },
            plugins: [
                "serverless-azure-functions"
            ],
            functions: functionMetadata || MockFactory.createTestFunctionsMetadata(2),
        };
        return (asYaml) ? js_yaml_1.default.dump(data) : data;
    };
    MockFactory.createTestFunctionsMetadata = function (functionCount) {
        if (functionCount === void 0) { functionCount = 2; }
        var data = {};
        for (var i = 0; i < functionCount; i++) {
            var functionName = "function" + (i + 1);
            data[functionName] = MockFactory.createTestFunctionMetadata(functionName);
        }
        return data;
    };
    MockFactory.createTestFunctionApimConfig = function (name) {
        return {
            apim: {
                operations: [
                    {
                        method: "get",
                        urlTemplate: name,
                        displayName: name,
                    },
                ],
            },
        };
    };
    MockFactory.createTestFunctionMetadata = function (name) {
        return {
            handler: "src/handlers/" + name + ".handler",
            events: [
                {
                    http: true,
                    "x-azure-settings": {
                        authLevel: "anonymous"
                    }
                },
                {
                    http: true,
                    "x-azure-settings": {
                        direction: "out",
                        name: "res"
                    },
                }
            ]
        };
    };
    MockFactory.createTestService = function (functions) {
        if (!functions) {
            functions = MockFactory.createTestSlsFunctionConfig();
        }
        var serviceName = "serviceName";
        return {
            getAllFunctions: jest.fn(function () { return Object.keys(functions); }),
            getFunction: jest.fn(),
            getAllEventsInFunction: jest.fn(),
            getAllFunctionsNames: jest.fn(function () { return Object.keys(functions); }),
            getEventInFunction: jest.fn(),
            getServiceName: jest.fn(function () { return serviceName; }),
            load: jest.fn(),
            mergeResourceArrays: jest.fn(),
            setFunctionNames: jest.fn(),
            update: jest.fn(),
            validate: jest.fn(),
            custom: null,
            provider: MockFactory.createTestAzureServiceProvider(),
            service: serviceName,
            artifact: "app.zip",
            functions: functions
        };
    };
    MockFactory.createTestFunctionsResponse = function (functions) {
        var e_1, _a;
        var result = [];
        functions = functions || MockFactory.createTestSlsFunctionConfig();
        try {
            for (var _b = __values(Object.keys(functions)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var name_1 = _c.value;
                result.push({ properties: MockFactory.createTestFunctionEnvelope(name_1) });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    };
    MockFactory.createTestAzureServiceProvider = function () {
        return {
            resourceGroup: "myResourceGroup",
            deploymentName: "myDeploymentName",
        };
    };
    MockFactory.createTestServicePrincipalEnvVariables = function () {
        return {
            azureSubId: "azureSubId",
            azureServicePrincipalClientId: "azureServicePrincipalClientId",
            azureServicePrincipalPassword: "azureServicePrincipalPassword",
            azureServicePrincipalTenantId: "azureServicePrincipalTenantId",
        };
    };
    MockFactory.createTestVariables = function () {
        return {
            azureCredentials: {
                tokenCache: {
                    _entries: [
                        {
                            accessToken: "token"
                        }
                    ]
                }
            },
            subscriptionId: "azureSubId",
        };
    };
    MockFactory.createTestSite = function (name) {
        if (name === void 0) { name = "Test"; }
        return {
            id: "appId",
            name: name,
            location: "West US",
            defaultHostName: "myHostName",
            enabledHostNames: [
                "myHostName"
            ]
        };
    };
    MockFactory.createTestFunctionEnvelope = function (name) {
        if (name === void 0) { name = "TestFunction"; }
        return {
            name: name,
            config: {
                bindings: MockFactory.createTestBindings()
            }
        };
    };
    MockFactory.createTestBindings = function (bindingCount) {
        if (bindingCount === void 0) { bindingCount = 3; }
        var bindings = [];
        for (var i = 0; i < bindingCount; i++) {
            bindings.push(MockFactory.createTestBinding());
        }
        return bindings;
    };
    MockFactory.createTestBinding = function () {
        // Only supporting HTTP for now, could support others
        return MockFactory.createTestHttpBinding();
    };
    MockFactory.createTestHttpBinding = function () {
        return {
            type: "httpTrigger",
            authLevel: "anonymous",
            direction: "in",
            methods: [
                "get",
                "post"
            ]
        };
    };
    MockFactory.createTestSlsFunctionConfig = function () {
        return {
            hello: __assign({}, MockFactory.createTestFunctionMetadata("hello"), MockFactory.createTestFunctionApimConfig("hello")),
            goodbye: __assign({}, MockFactory.createTestFunctionMetadata("goodbye"), MockFactory.createTestFunctionApimConfig("goodbye")),
        };
    };
    MockFactory.createTestApimService = function () {
        return {
            name: "APIM Service Instance",
            location: "West US",
            publisherName: "Somebody",
            publisherEmail: "somebody@example.com",
            sku: {
                capacity: 0,
                name: "Consumption",
            }
        };
    };
    MockFactory.createTestApimApi = function () {
        return {
            name: "Api1",
            path: "/api1",
        };
    };
    MockFactory.createTestUtils = function () {
        return {
            appendFileSync: jest.fn(),
            copyDirContentsSync: jest.fn(),
            dirExistsSync: jest.fn(function () { return false; }),
            fileExistsSync: jest.fn(function () { return false; }),
            findServicePath: jest.fn(),
            generateShortId: jest.fn(),
            getVersion: jest.fn(),
            logStat: jest.fn(),
            readFile: jest.fn(),
            readFileSync: jest.fn(function (filename) {
                if (filename === "serverless.yml") {
                    return MockFactory.createTestServerlessYml();
                }
            }),
            walkDirSync: jest.fn(),
            writeFile: jest.fn(),
            writeFileDir: jest.fn(),
            writeFileSync: jest.fn(),
        };
    };
    /**
     * Create a mock "request" module factory used to mock request objects that support piping
     * @param response The expected HTTP response
     */
    MockFactory.createTestMockRequestFactory = function (response) {
        if (response === void 0) { response = {}; }
        return jest.fn(function (options, callback) {
            setImmediate(function () { return callback(null, response); });
            // Required interface for .pipe()
            return {
                on: jest.fn(),
                once: jest.fn(),
                emit: jest.fn(),
                write: jest.fn(),
                end: jest.fn(),
            };
        });
    };
    MockFactory.createTestMockApiCorsPolicy = function () {
        return {
            allowCredentials: false,
            allowedOrigins: ["*"],
            allowedHeaders: ["*"],
            exposeHeaders: ["*"],
            allowedMethods: ["GET", "POST"],
        };
    };
    MockFactory.createTestCli = function () {
        return {
            log: jest.fn(),
        };
    };
    MockFactory.createTestPluginManager = function () {
        return {
            addPlugin: jest.fn(),
            cliCommands: null,
            cliOptions: null,
            commands: null,
            deprecatedEvents: null,
            hooks: null,
            loadAllPlugins: jest.fn(),
            loadCommand: jest.fn(),
            loadCommands: jest.fn(),
            loadCorePlugins: jest.fn(),
            loadPlugins: jest.fn(),
            loadServicePlugins: jest.fn(),
            plugins: null,
            serverless: null,
            setCliCommands: jest.fn(),
            setCliOptions: jest.fn(),
            spawn: jest.fn(),
        };
    };
    return MockFactory;
}());
exports.MockFactory = MockFactory;
